/*
 *  Copyright (C) 2004-2005 Mathias Andre <mathias.andre@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <string.h>
#include <libgnome/gnome-i18n.h>

#include "nb_interface.h"
#include "nb_notebook.h"
#include "nb_treeview.h"
#include "nb_noteview.h"

enum
{
	TARGET_STRING,
	TARGET_URL
};

static GtkTargetEntry drag_drop_target[] =
{
	{ "text/plain",    0, TARGET_STRING },
	{ "text/uri-list", 0, TARGET_URL },
};


static void
nb_interface_instance_init (GTypeInstance * instance, gpointer g_class)
{
	NbInterface * self = (NbInterface *) instance;
	self->dispose_has_run = FALSE;
}

static void
nb_interface_dispose (GObject * obj)
{
	NbInterface * self = (NbInterface *) obj;

	if ( self->dispose_has_run ) {
		/* If dispose did already run, return. */
		return;
	}

	/* Make sure dispose does not run twice. */
	self->dispose_has_run = TRUE;
}

static void
nb_interface_finalize (GObject * obj)
{
	NbInterface * self = (NbInterface *) obj;

	/* complete object destruction. */
}

static void
nb_interface_class_init (gpointer g_class, gpointer g_class_data)
{
	GObjectClass * gobject_class = G_OBJECT_CLASS (g_class);

	gobject_class->dispose = nb_interface_dispose;
	gobject_class->finalize = nb_interface_finalize;
}


GType
nb_interface_get_type (void)
{
	static GType type = 0;
	if ( type == 0 )
       	{
		static const GTypeInfo info = {
			sizeof (NbInterfaceClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			nb_interface_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (NbInterface),
			0,      /* n_preallocs */
			nb_interface_instance_init    /* instance_init */
		};
		type = g_type_register_static (G_TYPE_OBJECT,
				"NbInterfaceType",
				&info, 0);
	}
	return type;
}

NbInterface * 
nb_interface_new (void)
{
	NbInterface * self = NB_INTERFACE (g_object_new (NB_TYPE_INTERFACE, NULL));

	self->main_xml = glade_xml_new (g_build_path ("/", GLADE_DIR, "gnome-notebook.glade", NULL), "gnome-notebook", NULL);
	self->main_window =  glade_xml_get_widget (self->main_xml, "gnome-notebook");
	self->search_entry =  glade_xml_get_widget (self->main_xml, "entryfind");

	/* connect the signals in the interface */
	glade_xml_signal_autoconnect (self->main_xml);

	/* create the treeview and the noteview */
	self->treeview = nb_treeview_new (G_OBJECT (self));
	self->noteview = nb_noteview_new (G_OBJECT (self));

	/* set drag and drap on main_window */
	gtk_drag_dest_set (self->main_window, GTK_DEST_DEFAULT_ALL, drag_drop_target, 2, GDK_ACTION_COPY|GDK_ACTION_MOVE|GDK_ACTION_LINK);
	//TODO: SET up d&d g_signal_connect (self->main_window, "drag_data_received", G_CALLBACK (nb_ui_cb_drag_received), NULL);

	/* set the icon */
	gtk_window_set_icon ( GTK_WINDOW (self->main_window),
			gdk_pixbuf_new_from_file (g_build_path ("/", PIXMAPS_DIR, "gnome-notebook.svg", NULL), NULL));

	return self;
}

/* callbacks */
void
nb_interface_cb_new_note (GtkWidget * widget, gpointer userdata)
{
	extern NbNotebook * notebook;

	nb_notebook_new_note (notebook);
}

void
nb_interface_cb_delete_note (GtkWidget * widget, gpointer userdata)
{
	extern NbNotebook * notebook;

	nb_notebook_delete_note (notebook);
}

gboolean
nb_interface_cb_quit (GtkWidget * widget, gpointer userdata)
{
	extern NbNotebook * notebook;

	nb_notebook_quit (notebook);

	gtk_main_quit ();

	return TRUE;
}

void
nb_interface_cb_search_edited (GtkWidget * widget, gpointer userdata)
{
	extern NbNotebook * notebook;

	nb_treeview_update_with_query (notebook->ui->treeview, (gchar *) gtk_entry_get_text ( GTK_ENTRY (widget)));
}

void
nb_interface_cb_search_cleared (GtkWidget * widget, gpointer userdata)
{
	extern NbNotebook * notebook;

	gtk_entry_set_text ( GTK_ENTRY (notebook->ui->search_entry), "");
}

void
nb_interface_cb_about_dialog (GtkWidget * widget, gpointer userdata)
{
	const gchar * authors[] = { "Mathias Andre <mathias.andre@gmail.com>", NULL};
	const gchar * comment = (gchar *) _("Notebook is a simple logbook editor.");
	const gchar * translator = (gchar *) _("Mathias Andre <mathias.andre@gmail.com>");

	GdkPixbuf * logo = gdk_pixbuf_new_from_file_at_size (g_build_path ("/", PIXMAPS_DIR, "gnome-notebook.svg", NULL), 200, 200, NULL);	
	g_print ("path:%s\n", g_build_path ("/", PIXMAPS_DIR, "notebook.svg", NULL));
	GtkWidget * about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
			"name", "GNOME Notebook", 
			"version", "0.4",
			"copyright", "(C) 2004-2005 Mathias Andre",
			"comments", "Notebook is a simple logbook editor.",
			"authors", authors,
			"translator-credits", _("translator-credits"),
			"logo", logo,
			NULL);

	gtk_widget_show (about);

	g_object_unref (G_OBJECT (logo));
}

/* drag and drop callbacks */
void
nb_ui_cb_drag_received (GtkWidget * widget, GdkDragContext * context, int x, int y, GtkSelectionData * seldata, guint info, guint time, gpointer userdata){
	extern NbNotebook * notebook;
	gchar ** split, ** copy ;
	GSList * attachments = NULL;

	if ( seldata == NULL )
		return ;
	if ( seldata->length < 0 )
		return ;

	split = g_strsplit ((gchar *) seldata->data, "\n", 0);

	for ( copy = split ; *copy != NULL ; copy++ )
	{
		if ( strlen (*copy) > 0 )
			attachments = g_slist_append (attachments, g_strchomp (*copy));
	}

	//nb_global_add_attachments (g, attachments);

	gtk_drag_finish (context, TRUE, FALSE, time);
	g_signal_stop_emission_by_name (widget, "drag_data_received");
}

