/*
 *  Copyright (C) 2004-2005 Mathias Andre <mathias.andre@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <time.h>
#include <glib.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnome/gnome-i18n.h>
#include <libxml/tree.h>

#include "nb_notebook.h"
#include "nb_util.h"

static void
nb_note_instance_init (GTypeInstance * instance, gpointer g_class)
{
	NbNote * self = (NbNote *) instance;
	self->title = NULL;
	self->text = NULL;
	self->creation_date = 0;
	self->last_update = 0;
	nb_note_set_creation_date (self);
	nb_note_set_last_update (self);
	self->type = g_strdup (_("Default"));
	self->attachments = g_ptr_array_new ();
	self->iter = NULL;
	self->dispose_has_run = FALSE;
}

static void
nb_note_dispose (GObject * obj)
{
	NbNote * self = (NbNote *) obj;

	if ( self->dispose_has_run ) {
		/* If dispose did already run, return. */
		return;
	}

	/* Make sure dispose does not run twice. */
	self->dispose_has_run = TRUE;
}

static void
nb_note_finalize (GObject * obj)
{
	NbNote * self = (NbNote *) obj;

	/* complete object destruction. */
	g_free (self->title);
	g_free (self->text);
	g_ptr_array_free (self->attachments, TRUE);
}

static void
nb_note_class_init (gpointer g_class, gpointer g_class_data)
{
	GObjectClass * gobject_class = G_OBJECT_CLASS (g_class);

	gobject_class->dispose = nb_note_dispose;
	gobject_class->finalize = nb_note_finalize;
}


GType
nb_note_get_type (void)
{
	static GType type = 0;
	if ( type == 0 )
       	{
		static const GTypeInfo info = {
			sizeof (NbNoteClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			nb_note_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (NbNote),
			0,      /* n_preallocs */
			nb_note_instance_init    /* instance_init */
		};
		type = g_type_register_static (G_TYPE_OBJECT,
				"NbNoteType",
				&info, 0);
	}
	return type;
}

NbNote *
nb_note_new (void)
{
	return NB_NOTE (g_object_new (NB_TYPE_NOTE, NULL));
}

/* methods */
void
nb_note_set_title (NbNote * self, gchar * title)
{
	g_free (self->title);
	self->title = title;
}

void
nb_note_set_text (NbNote * self, gchar * text)
{
	g_free (self->text);
	self->text = text;
}

void
nb_note_set_type (NbNote * self, gchar * type)
{
	extern NbNotebook * notebook;

	if ( strcmp (self->type, type) == 0 )
		return ;

	nb_notebook_remove_note_type (notebook, self->type);
	nb_notebook_add_note_type (notebook, type);

	g_free (self->type);
	self->type = type;
}

void
nb_note_set_creation_date (NbNote * self)
{
	self->creation_date = time (NULL);
}

void
nb_note_set_last_update  (NbNote * self)
{
	self->last_update = time (NULL);
}

void
nb_note_clear_text (NbNote * self)
{
	g_free (self->text);
	self->text = NULL;
}


gchar *
nb_note_get_utf_title (NbNote * self)
{
	gsize bytes_written;

	if ( !self->title )
		return g_strdup ("");

	return  g_locale_to_utf8 (self->title, -1, NULL, &bytes_written, NULL);
}

gchar *
nb_note_get_utf_type (NbNote * self)
{
	gsize bytes_written;
	if ( !self->type )
		return g_strdup ("");

	return  g_locale_to_utf8 (self->type, -1, NULL, &bytes_written, NULL);
}

gchar *
nb_note_get_utf_creation_date (NbNote * self)
{
	gsize bytes_written;
	gchar * creation_date_utf, * creation_date;
	GDate * date = g_date_new ();
	g_date_set_time (date, self->creation_date);

	creation_date = nb_util_get_date_string (date);
	creation_date_utf = g_locale_to_utf8 (creation_date, -1, NULL, &bytes_written, NULL);
	g_free (creation_date);
	g_date_free (date);

	return creation_date_utf;
}

gchar *
nb_note_get_utf_last_update (NbNote * self)
{
	gsize bytes_written;
	gchar * last_update_utf, * last_update;
	GDate * date = g_date_new ();
	g_date_set_time (date, self->last_update);

	last_update = nb_util_get_date_string (date);
	last_update_utf = g_locale_to_utf8 (last_update, -1, NULL, &bytes_written, NULL);
	g_free (last_update);
	g_date_free (date);

	return last_update_utf;
}

gchar *
nb_note_get_utf_text (NbNote * self)
{
	if ( !self->text )
		return g_strdup ("");

	return  g_locale_to_utf8 (self->text, -1, NULL, NULL, NULL);

}


/* attachments */
void
nb_note_add_attachment (NbNote * self, NbAttachment * attachment)
{
	/* TODO: check if the attachment uri already exists */
	g_return_if_fail (attachment != NULL);
	g_ptr_array_add (self->attachments, attachment);
}

void
nb_note_remove_attachment (NbNote * self, NbAttachment * attachment)
{
	g_return_if_fail (attachment != NULL);
	g_ptr_array_remove (self->attachments, attachment);
}

gint
nb_note_get_nb_attachments (NbNote * self)
{
	return self->attachments->len;
}

void
nb_note_check_attachments (NbNote * self)
{
	gint i;
	NbAttachment * attachment;

	for ( i = 0 ; i < self->attachments->len ; i++ )
	{
		attachment = g_ptr_array_index (self->attachments, i);
		if ( !nb_attachment_check (attachment))
			nb_note_remove_attachment (self, attachment);
	}

}

xmlNodePtr
nb_note_get_xml_pointer (NbNote * self, gint index)
{
	xmlNodePtr cur;
	gchar * cindex, * ccreation_date, * clast_update;

	cur = xmlNewNode (NULL, "note");

	cindex = g_strdup_printf ("%d", index);
	xmlNewProp (cur, "index", (xmlChar *) cindex);

	xmlNewTextChild (cur, NULL, "type", self->type);
	xmlNewTextChild (cur, NULL, "title", self->title);
	xmlNewTextChild (cur, NULL, "text", self->text);
	ccreation_date = g_strdup_printf ("%d", self->creation_date);
	clast_update = g_strdup_printf ("%d", self->last_update);
	xmlNewTextChild (cur, NULL, "creation_date", ccreation_date);
	xmlNewTextChild (cur, NULL, "last_update", clast_update);

	/* TODO: save date to file */

	g_free (cindex);
	g_free (ccreation_date);
	g_free (clast_update);

	return cur;
}

void
nb_note_load_text (NbNote * self, xmlDocPtr doc, xmlNodePtr cur)
{
	cur = cur->xmlChildrenNode;

	xmlChar * text = NULL;

	while ( cur != NULL )
	{
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "text") )
		{
			text = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);
			break;
		}

		cur = cur->next;
	}

	nb_note_set_text (self, text);
}

NbNote *
nb_note_new_from_xml_doc (xmlDocPtr doc, xmlNodePtr cur)
{
	NbNote * note = nb_note_new ();

	cur = cur->xmlChildrenNode;

	while ( cur != NULL )
	{
		/* get the title */
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "title") )
			nb_note_set_title (note, xmlNodeListGetString (doc, cur->xmlChildrenNode, 1));

		/* get the type */
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "type") )
			nb_note_set_type (note, xmlNodeListGetString (doc, cur->xmlChildrenNode, 1));

		/* get the creation date */
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "creation_date") )
			note->creation_date = atoi (xmlNodeListGetString (doc, cur->xmlChildrenNode, 1));

		/* get the lastupdate */
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "last_update") )
			note->creation_date = atoi (xmlNodeListGetString (doc, cur->xmlChildrenNode, 1));

		/* get the attachments */
		/* TODO: 
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "attachment") )
			nb_note_add_attachment (n, nb_attachment_new_from_xml_doc (doc, cur));
		*/

		cur = cur->next;
	}

	return note;
}

/* queries */
gboolean
nb_note_matches_query (NbNote * self, gchar * query)
{
	if ( g_strrstr ( g_utf8_casefold (self->title, -1), query) != NULL )
		return TRUE;

	if ( ( self->text != NULL ) && ( g_strrstr (g_utf8_casefold (self->text, -1), query) != NULL ) )
		return TRUE;

	return FALSE;
}

