/*
 *  Copyright (C) 2004-2005 Mathias Andre <mathias.andre@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <libgnome/gnome-i18n.h>
#include <libxml/tree.h>
#include <stdio.h>
#include <errno.h>

#include "nb_notebook.h"

static void nb_notebook_add_note (NbNotebook * self, NbNote * note);
static void nb_notebook_remove_note (NbNotebook * self, NbNote * note);
static gboolean nb_notebook_parse_xml_file (NbNotebook * self, xmlDocPtr * doc, xmlNodePtr * cur);
static gboolean nb_notebook_cb_delete_note_from_file (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, GList ** rowref_list);

static void
nb_notebook_instance_init (GTypeInstance * instance, gpointer g_class)
{
	NbNotebook * self = (NbNotebook *) instance;
	self->ui = NULL;
	self->current_note = NULL;
	self->last_selected_note = NULL;
	self->notes = g_ptr_array_new ();
	self->note_types = g_ptr_array_new ();
	self->note_is_inserted = FALSE;
	self->note_is_modified = FALSE;
	self->note_edited_lock = FALSE;
	/* TODO: create the .notebook directory if it doesn't already exist */
	self->filename = NULL;
	self->dispose_has_run = FALSE;
}

static void
nb_notebook_dispose (GObject * obj)
{
	NbNotebook * self = (NbNotebook *) obj;

	if ( self->dispose_has_run ) {
		/* If dispose did already run, return. */
		return;
	}

	/* Make sure dispose does not run twice. */
	self->dispose_has_run = TRUE;
}

static void
nb_notebook_finalize (GObject * obj)
{
	NbNotebook * self = (NbNotebook *) obj;

	/* complete object destruction. */
}

static void
nb_notebook_class_init (gpointer g_class, gpointer g_class_data)
{
	GObjectClass * gobject_class = G_OBJECT_CLASS (g_class);

	gobject_class->dispose = nb_notebook_dispose;
	gobject_class->finalize = nb_notebook_finalize;
}


GType
nb_notebook_get_type (void)
{
	static GType type = 0;
	if ( type == 0 )
       	{
		static const GTypeInfo info = {
			sizeof (NbNotebookClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			nb_notebook_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (NbNotebook),
			0,      /* n_preallocs */
			nb_notebook_instance_init    /* instance_init */
		};
		type = g_type_register_static (G_TYPE_OBJECT,
				"NbNotebookType",
				&info, 0);
	}
	return type;
}

NbNotebook *
nb_notebook_new (void)
{
	NbNotebook * self = NB_NOTEBOOK (g_object_new (NB_TYPE_NOTEBOOK, NULL));
	self->ui = nb_interface_new ();

	/* create the default directory if they don't exist */
	gchar * configpath = g_strdup_printf ("%s/.gnome2/gnome-notebook", g_get_home_dir ());

	if ( mkdir ( configpath, 0777) == -1 )
	{
		if ( errno != EEXIST )
		{
			perror ("Can't create default directory: ");
			exit (EXIT_FAILURE);
		}
	}

	g_free (configpath);

	self->filename = g_strdup_printf ("%s/.gnome2/gnome-notebook/notes.xml", g_get_home_dir ());

	nb_notebook_add_note_type (self, _("Default"));

	return self;

}

/* methods */
void
nb_notebook_quit (NbNotebook * self)
{
	nb_notebook_save_note (self);
}

void
nb_notebook_new_note (NbNotebook * self)
{
	nb_notebook_save_note (self);

	/* free the current note text to gain memory */
	if ( self->current_note != NULL )
		nb_note_clear_text (self->current_note);

	/* create a new blank note */
	self->current_note = nb_note_new ();

	/* TODO: update attachment view */

	/* add note to note_list */
	nb_notebook_add_note (self, self->current_note);
	/* add note to treeview */
	nb_treeview_add_note (self->ui->treeview, self->current_note, TRUE);
	self->note_is_inserted = FALSE;
	self->note_is_modified = FALSE;

}

void
nb_notebook_delete_note (NbNotebook * self)
{
	GtkTreeSelection * selection = nb_treeview_get_selection (self->ui->treeview);
	gint count = gtk_tree_selection_count_selected_rows (selection);

	if ( count < 1 ) 
		return ;

	/* TODO: create confirmation dialog 
	if ( count == 1 )
	{
		if ( !nb_ui_dialog_question ((gchar *) _("Do you want to delete current note?"), (gchar *) _("Delete note?")) )
			return ;
	}
	else
	{
		if ( !nb_ui_dialog_question ((gchar *) _("Do you want to delete all selected notes?"), (gchar *) _("Delete note?")) )
			return ;
	}*/

	GList * rr_list = NULL;    /* list of GtkTreeRowReferences to remove */
	GList * node;

	/* delete note(s) from file  and get list of notes' pointers */
	gtk_tree_selection_selected_foreach (selection, (GtkTreeSelectionForeachFunc) nb_notebook_cb_delete_note_from_file, &rr_list);

	nb_treeview_remove_note_list (self->ui->treeview, rr_list);

	/* TODO: update status bar 
	if ( count == 1 )
		nb_ui_statusbar_push ((gchar *) _("Note successfully deleted"));
	else
		nb_ui_statusbar_push(g_strdup_printf ((gchar *) _("%d notes successfully deleted"), count));
	*/
}

void
nb_notebook_save_note (NbNotebook * self)
{
	if ( self->note_edited_lock == TRUE )
		return ;

	if ( self->current_note == NULL )
		return ;

	if ( !self->note_is_modified )
		return ;

	if ( self->note_is_inserted )
	{
		nb_note_set_last_update (self->current_note);
		/* update the last update field in the treeview */
		nb_treeview_update_note (self->ui->treeview, self->current_note);
		nb_notebook_update_xml_file (self, UPDATE_NOTE);
	}
	else
		nb_notebook_update_xml_file (self, ADD_NOTE);
	
	if ( !self->note_is_inserted )
		self->note_is_inserted = TRUE;

	self->note_is_modified = FALSE;
}

static void
nb_notebook_add_note (NbNotebook * self, NbNote * note)
{
	g_ptr_array_add (self->notes, note);
}

static void
nb_notebook_remove_note (NbNotebook * self, NbNote * note)
{
	g_ptr_array_remove (self->notes, note);
	nb_notebook_remove_note_type (self, note->type);
}

gint
nb_notebook_get_note_index (NbNotebook * self, NbNote * note)
{
	gint index;

	for ( index = 0 ; index < self->notes->len ; index++ )
	{
		if ( g_ptr_array_index (self->notes, index) == note )
			return index;
	}

	return -1;
}

void
nb_notebook_add_note_type (NbNotebook * self, gchar * ctype)
{
	NbNoteType * type;
	gint index;

	/* check if the type already exists */
	for ( index = 0 ; index < self->note_types->len ; index++ )
	{
		type = (NbNoteType *) g_ptr_array_index (self->note_types, index); 

		/* if the type exists increase the number of occurences */
		if ( strcmp (type->name, ctype) == 0 )
		{
			(type->nb)++;
			return ;
		}
	}

	/* it the type doesn't exit add it the list */
	type = g_malloc (sizeof (NbNoteType));
	type->name = g_strdup (ctype);
	if ( strcmp (ctype, _("Default")) == 0 )
		type->nb = -1; /* prevent Default type to be ever deleted */
	else
		type->nb = 1;

	g_ptr_array_add (self->note_types, type);
	nb_noteview_add_combo_entry (self->ui->noteview, ctype);
}

void
nb_notebook_remove_note_type (NbNotebook * self, gchar * ctype)
{
	NbNoteType * type;
	gint index;
	
	for ( index = 0 ; index < self->note_types->len ; index++ )
	{
		type = (NbNoteType *) g_ptr_array_index (self->note_types, index); 

		/* if the type exists decrease the number of occurences */
		if ( strcmp (type->name, ctype) == 0 )
		{
			(type->nb)--;

			/*if last occurence of type remove the type */
			if ( type->nb == 0 )
			{
				g_free (type->name);
				g_ptr_array_remove_index (self->note_types, index);
				nb_noteview_remove_combo_entry (self->ui->noteview, index);
			}

			return ;
		}

	}
}

gint
nb_notebook_get_note_type_index (NbNotebook * self, gchar * ctype)
{
	NbNoteType * type;
	gint index;

	for ( index = 0 ; index < self->note_types->len ; index++ )
	{
		type = (NbNoteType *) g_ptr_array_index (self->note_types, index); 

		if (strcmp (type->name, ctype) == 0 )
			return index;
	}

	return -1;
}

gint
nb_notebook_get_nb_notes (NbNotebook * self)
{
	return self->notes->len;
}

void
nb_notebook_load_xml_file (NbNotebook * self)
{
	xmlDocPtr doc;
	xmlNodePtr cur;

	if ( !nb_notebook_parse_xml_file (self, &doc, &cur) )
		return ;

	/* loop every "note" nodes */
	while ( cur != NULL )
	{
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "note") )
		{
			NbNote * note = NULL;
			note = nb_note_new_from_xml_doc (doc, cur);

			if ( note == NULL )
				g_warning ("Invalid note in file, skip note...\n");
			else
			{
				nb_notebook_add_note (self, note);
			}
		}
		cur = cur->next;
	}

	xmlFreeDoc (doc);
}

void
nb_notebook_load_note_text (NbNotebook * self, NbNote * note)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	gint index;

	if ( !nb_notebook_parse_xml_file (self, &doc, &cur) )
		return ;

	index = nb_notebook_get_note_index (self, note);

	while ( cur != NULL )
	{
		if ( !xmlStrcmp (cur->name, (const xmlChar *) "note") )
		{
			gint i = atoi (xmlGetProp (cur, "index"));

			if ( i == index )
			{
				nb_note_load_text (note, doc, cur);

				xmlFreeDoc (doc);

				return ;
			}
		}
		cur = cur->next;
	}

	xmlFreeDoc (doc);
}

void
nb_notebook_update_xml_file (NbNotebook * self, NbXmlAction action)
{
	xmlDocPtr doc;
	xmlNodePtr cur;

	doc = xmlParseFile (self->filename);

	if ( doc == NULL )
	{
		if ( action == ADD_NOTE )
		{
			g_print ("No document found, creating one.\n");

			/* document is empty */
			doc = xmlNewDoc ("1.0");

			cur = xmlNewNode (NULL, "notebook");

			xmlDocSetRootElement (doc, cur);
		}
		else
		{
			g_warning ("Document not parsed successfully.\n");
			return ;
		}
	}

	cur = xmlDocGetRootElement (doc);

	if ( cur == NULL )
	{
		g_warning ("Empty document.\n");
		xmlFreeDoc (doc);
		return ;
	}

	if ( xmlStrcmp (cur->name, (const xmlChar *) "notebook") )
	{
		g_warning ("Wrong document type, root node is not \"notebook\".\n");
		xmlFreeDoc (doc);
		return ;
	}

	gint index = nb_notebook_get_note_index (self, self->current_note);
	gint i;

	if ( action == ADD_NOTE )
		xmlAddChild (cur, nb_note_get_xml_pointer (self->current_note, index));
	else
	{
		gboolean reindex = FALSE;

		cur = cur->xmlChildrenNode;

		/* loop every "note" node */
		while ( cur != NULL )
		{
			if ( !xmlStrcmp (cur->name, (const xmlChar *) "note") )
			{

				i = atoi (xmlGetProp (cur, "index"));

				if ( reindex )
				{
					/* reindex the following notes after one has been removed */
					gchar * cindex = g_strdup_printf ("%d", i -1);

					xmlSetProp (cur, "index", cindex);
					g_free (cindex);
				}
				else if ( i == index )
				{
					if ( action == UPDATE_NOTE )
					{
						xmlReplaceNode (cur, nb_note_get_xml_pointer (self->current_note, index));
						break ;
					}
					else if ( action == REMOVE_NOTE )
					{
						xmlNodePtr del = cur;

						if ( cur->prev != NULL )
							cur->prev->next = cur->next;
						if ( cur->next != NULL )
							cur->next->prev = cur->prev;

						xmlFreeNode (del);
						reindex = TRUE;
					}
				}
			}
			cur = cur->next;
		}
	}

	xmlSaveFile (self->filename, doc);

	xmlFreeDoc (doc);
}

static gboolean
nb_notebook_parse_xml_file (NbNotebook * self, xmlDocPtr * doc, xmlNodePtr * cur)
{
	* doc = xmlParseFile (self->filename);

	if ( * doc == NULL )
	{
		g_warning ("Document not parsed successfully.\n");
		return FALSE;
	}

	* cur = xmlDocGetRootElement (* doc);

	if ( * cur == NULL )
	{
		g_warning  ("Empty document.\n");
		xmlFreeDoc (* doc);
		return FALSE;
	}

	if ( xmlStrcmp ((* cur)->name, (const xmlChar *) "notebook") )
	{
		g_warning ("Wrong document type, root node is not \"notebook\".\n");
		xmlFreeDoc (* doc);
		return FALSE;
	}

	* cur = (* cur)->xmlChildrenNode;

	return TRUE;
}

/* callbacks */
static gboolean
nb_notebook_cb_delete_note_from_file (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, GList ** rowref_list)
{
	NbNote * note;
	extern NbNotebook * notebook;

	g_assert ( rowref_list != NULL );

	gtk_tree_model_get (model, iter, NOTE_POINTER_COLUMN, &note, -1);

	nb_note_set_text (notebook->current_note, NULL);

	/* set the current note as it's the one being deleted by nb_notebook_update_xml_file */
	notebook->current_note = note;

	/* delete the note from the xml file */
	nb_notebook_update_xml_file (notebook, REMOVE_NOTE);

	GtkTreeRowReference * rowref;

	rowref = gtk_tree_row_reference_new (model, path);
	* rowref_list = g_list_append (* rowref_list, rowref);

	/* remove the note from the notelist */
	nb_notebook_remove_note (notebook, note);

	return FALSE;
}
