/*
 *  Copyright (C) 2004-2005 Mathias Andre <mathias.andre@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <libgnomeui/libgnomeui.h>
#include <libgnome/gnome-i18n.h>

#include "nb_treeview.h"
#include "nb_notebook.h"
#include "nb_interface.h"

/* column ordering callbacks */
static void	nb_treeview_remove_note (NbTreeview * self, GtkTreeIter * iter);
static gint     nb_treeview_cb_sort_iter_compare_string (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gpointer userdata);
static gint     nb_treeview_cb_sort_iter_compare_date (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gpointer userdata);
static void	nb_treeview_cb_update_notebook_with_current_note (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer user_data);
static void	nb_treeview_select_iter (NbTreeview * self, GtkTreeIter * iter);
static gboolean	nb_treeview_cb_row_matches_query (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data);

static void	nb_treeview_cb_drag_begin (GtkWidget * widget, GdkDragContext * context, gpointer user_data);
static gboolean	nb_treeview_cb_button_release (GtkWidget * widget, GdkEventButton * event, gpointer user_data);

static void	nb_treeview_set_sort_func (NbTreeview * self);

static void
nb_treeview_instance_init (GTypeInstance * instance, gpointer g_class)
{
	NbTreeview * self = (NbTreeview *) instance;
	self->dispose_has_run = FALSE;
	self->tree = NULL;
	self->store =  gtk_list_store_new (N_COLUMNS,
			    		   G_TYPE_STRING,
					   G_TYPE_STRING,
					   G_TYPE_STRING,
					   G_TYPE_STRING,
					   G_TYPE_STRING,
					   G_TYPE_POINTER);
}

static void
nb_treeview_dispose (GObject * obj)
{
	NbTreeview * self = (NbTreeview *) obj;

	if ( self->dispose_has_run ) {
		/* If dispose did already run, return. */
		return;
	}

	/* Make sure dispose does not run twice. */
	self->dispose_has_run = TRUE;
}

static void
nb_treeview_finalize (GObject * obj)
{
	NbTreeview * self = (NbTreeview *) obj;

	/* complete object destruction. */
}

static void
nb_treeview_class_init (gpointer g_class, gpointer g_class_data)
{
	GObjectClass * gobject_class = G_OBJECT_CLASS (g_class);

	gobject_class->dispose = nb_treeview_dispose;
	gobject_class->finalize = nb_treeview_finalize;
}


GType
nb_treeview_get_type (void)
{
	static GType type = 0;
	if ( type == 0 )
       	{
		static const GTypeInfo info = {
			sizeof (NbTreeviewClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			nb_treeview_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (NbTreeview),
			0,      /* n_preallocs */
			nb_treeview_instance_init    /* instance_init */
		};
		type = g_type_register_static (G_TYPE_OBJECT,
				"NbTreeviewType",
				&info, 0);
	}
	return type;
}

NbTreeview *
nb_treeview_new (GObject * interface)
{
	NbTreeview * self = NB_TREEVIEW (g_object_new (NB_TYPE_TREEVIEW, NULL));
	extern NbNotebook * notebook;
	GtkTreeViewColumn * column;
	GtkCellRenderer * renderer;
	GtkTreeSelection * selection;
	GtkWidget * image;
	NbInterface * ui = NB_INTERFACE (interface);

	self->tree = glade_xml_get_widget (ui->main_xml, "treeview");

	gtk_tree_view_set_model (GTK_TREE_VIEW (self->tree), GTK_TREE_MODEL (self->store));

	nb_treeview_set_sort_func (self);

	renderer = gtk_cell_renderer_pixbuf_new ();
	column = gtk_tree_view_column_new ();
	gtk_tree_view_column_pack_start (column, renderer, FALSE);
	gtk_tree_view_column_set_attributes (column, renderer, "stock-id", ATTACHMENT_COLUMN, NULL);
	image = gtk_image_new_from_stock (GNOME_STOCK_ATTACH, GTK_ICON_SIZE_MENU);
	gtk_tree_view_column_set_widget (column, image);
	gtk_tree_view_column_set_max_width (column, 20);
	gtk_widget_show (image);
	gtk_tree_view_append_column (GTK_TREE_VIEW(self->tree), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ((gchar *) _("Title"), renderer,
			"text", TITLE_COLUMN,
			NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sort_column_id (column, TITLE_COLUMN);
	gtk_tree_view_append_column (GTK_TREE_VIEW (self->tree), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ((gchar *) _("Type"), renderer,
			"text", TYPE_COLUMN,
			NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sort_column_id (column, TYPE_COLUMN);
	gtk_tree_view_append_column (GTK_TREE_VIEW (self->tree), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ((gchar *) _("Creation date"), renderer,
			"text", CREATION_DATE_COLUMN,
			NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sort_indicator (column, TRUE);
	gtk_tree_view_column_set_sort_column_id (column, CREATION_DATE_COLUMN);
	gtk_tree_view_append_column (GTK_TREE_VIEW (self->tree), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ((gchar *) _("Last update"), renderer,
			"text", LAST_UPDATE_COLUMN,
			NULL);
	gtk_tree_view_column_set_resizable (column, TRUE);
	gtk_tree_view_column_set_sort_column_id (column, LAST_UPDATE_COLUMN);
	gtk_tree_view_append_column (GTK_TREE_VIEW (self->tree), column);

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (self->tree));
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (nb_treeview_cb_selection_changed), NULL);
	g_signal_connect (GTK_WIDGET (self->tree), "button-release-event", G_CALLBACK (nb_treeview_cb_button_release), NULL);
	g_signal_connect (GTK_WIDGET (self->tree), "drag-begin", G_CALLBACK (nb_treeview_cb_drag_begin), NULL);

	/* Drag and Drop */
	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (self->tree),
			GDK_BUTTON1_MASK,
			target_list,
			1,
			GDK_ACTION_LINK);
	
	return self;
}

/* methods */
GtkTreeIter *
nb_treeview_add_note (NbTreeview * self, NbNote * n, gboolean scroll)
{
	GtkTreeIter * iter = g_malloc (sizeof (GtkTreeIter));
	gchar * title, * type, * creation_date, * last_update;

	/* Acquire an iterator */
	gtk_list_store_append ( GTK_LIST_STORE (self->store) , iter);

	title = nb_note_get_utf_title (n);
	type = nb_note_get_utf_type (n);
	creation_date = nb_note_get_utf_creation_date (n);
	last_update = nb_note_get_utf_last_update (n);

	/* convert the strings to utf8 and insert them */
	gtk_list_store_set ( GTK_LIST_STORE (self->store) , iter,
			TITLE_COLUMN, title,
			TYPE_COLUMN, type,
			CREATION_DATE_COLUMN, creation_date,
			LAST_UPDATE_COLUMN, last_update,
			NOTE_POINTER_COLUMN, n,
			-1);

	/* TODO update attachment icon */

	n->iter = iter;

	if ( scroll )
	{
		/* scroll and select the added note */
		gtk_tree_view_scroll_to_cell ( GTK_TREE_VIEW (self->tree),
				gtk_tree_model_get_path ( gtk_tree_view_get_model (GTK_TREE_VIEW (self->tree)), iter),
				NULL, FALSE, 0.0, 0.0);

		nb_treeview_select_iter (self, iter);
	}


	g_free (title);
	g_free (type);
	g_free (creation_date);
	g_free (last_update);
}

void
nb_treeview_update_note (NbTreeview * self, NbNote * n)
{
	gchar * title, * type, * creation_date, * last_update;

	if ( n->iter == NULL )
		return ;

	title = nb_note_get_utf_title (n);
	type = nb_note_get_utf_type (n);
	creation_date = nb_note_get_utf_creation_date (n);
	last_update = nb_note_get_utf_last_update (n);

	gtk_list_store_set ( GTK_LIST_STORE (self->store), n->iter,
			TITLE_COLUMN, title,
			TYPE_COLUMN, type,
			CREATION_DATE_COLUMN, creation_date,
			LAST_UPDATE_COLUMN, last_update,
			NOTE_POINTER_COLUMN, n,
			-1);
	/* TODO update attachment icon */

	g_free (title);
	g_free (type);
	g_free (creation_date);
	g_free (last_update);
}

void
nb_treeview_update_attachment_icon (NbTreeview * self, NbNote * n)
{
	if ( n->iter == NULL )
		return ;

	/* if the note has attachments then display an icon in the treeview */
	if (  nb_note_get_nb_attachments (n) == 0 )
	{
		gtk_list_store_set ( GTK_LIST_STORE (self->store), n->iter,
				ATTACHMENT_COLUMN, NULL,
				-1);

		return ;
	}

	gtk_list_store_set ( GTK_LIST_STORE (self->store), n->iter,
			ATTACHMENT_COLUMN, GNOME_STOCK_ATTACH,
			-1);
}

GtkTreeSelection *
nb_treeview_get_selection (NbTreeview * self)
{
	return gtk_tree_view_get_selection ( GTK_TREE_VIEW (self->tree));
}

static void
nb_treeview_remove_note (NbTreeview * self, GtkTreeIter * iter)
{
	gtk_list_store_remove(self->store, iter);
}

void
nb_treeview_remove_note_list (NbTreeview * self, GList * rr_list)
{
	GList * node;
	GtkTreePath * path;

	for ( node = rr_list;  node != NULL;  node = node->next )
	{
		path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *) node->data);

		if ( path )
		{
			GtkTreeIter iter;

			if ( gtk_tree_model_get_iter ( GTK_TREE_MODEL(self->store), &iter, path) )
			{
				nb_treeview_remove_note (self, &iter);
			}
		}
	}
}

/* private methods */
static void
nb_treeview_select_iter (NbTreeview * self, GtkTreeIter * iter)
{
	GtkTreeModel * model;
	GtkTreeSelection * selection;

	if ( iter == NULL )
		return ;

	model = gtk_tree_view_get_model ( GTK_TREE_VIEW (self->tree));
	selection = gtk_tree_view_get_selection ( GTK_TREE_VIEW (self->tree));

	gtk_tree_selection_unselect_all (selection);
	gtk_tree_selection_select_iter (selection, iter);
}

static void
nb_treeview_set_sort_func (NbTreeview * self)
{
	GtkTreeSortable * sortable;

	sortable = GTK_TREE_SORTABLE (self->store);

	gtk_tree_sortable_set_sort_func (sortable, TITLE_COLUMN, nb_treeview_cb_sort_iter_compare_string, GINT_TO_POINTER (TITLE_COLUMN), NULL);
	gtk_tree_sortable_set_sort_func (sortable, TYPE_COLUMN, nb_treeview_cb_sort_iter_compare_string, GINT_TO_POINTER (TYPE_COLUMN), NULL);
	gtk_tree_sortable_set_sort_func (sortable, CREATION_DATE_COLUMN, nb_treeview_cb_sort_iter_compare_date, GINT_TO_POINTER (CREATION_DATE_COLUMN), NULL);
	gtk_tree_sortable_set_sort_func (sortable, LAST_UPDATE_COLUMN, nb_treeview_cb_sort_iter_compare_date, GINT_TO_POINTER (LAST_UPDATE_COLUMN), NULL);

}

void
nb_treeview_update_with_query (NbTreeview * self, gchar * query)
{
	extern NbNotebook * notebook;
	static gchar * old_query = NULL;
	NbNote * note;
	gint i;


	if ( query != NULL )
		query = g_utf8_casefold (query, -1);

	if ( (query != NULL) && (old_query != NULL) )
	{
		/* if the new old query is part of the new one, then we only need to check the currently displayed notes */
		if ( g_strrstr (query, old_query) != NULL )
		{
			GList * node;
			RowQueryList parm;
			parm.rr_list = NULL;
			parm.query = query;

			gtk_tree_model_foreach ( GTK_TREE_MODEL (self->store), nb_treeview_cb_row_matches_query, &parm);

			for ( node = parm.rr_list;  node != NULL;  node = node->next )
			{
				GtkTreePath * path;
				path = gtk_tree_row_reference_get_path ( (GtkTreeRowReference *) node->data);

				if ( path )
				{
					GtkTreeIter iter;

					if ( gtk_tree_model_get_iter ( GTK_TREE_MODEL (self->store), &iter, path))
					{
						gtk_list_store_remove (self->store, &iter);
					}
				}
			}

			old_query = query;

			return ;
		}

	}

	/* clear the treeview */
	gtk_list_store_clear (self->store);

	for ( i = 0 ; i < nb_notebook_get_nb_notes (notebook) ; i++ )
	{
		note = g_ptr_array_index (notebook->notes, i);

		if ( query != NULL )
			nb_notebook_load_note_text (notebook, note);

		if ( (query == NULL) || (nb_note_matches_query (note, query)) )
		{
			nb_note_set_text (note, NULL);
			//notebook->note_edited_lock = TRUE;
			nb_treeview_add_note (self, note, TRUE);
			//notebook->note_edited_lock = FALSE;
		}
	}

	old_query = query;
}

/* callbacks */
void
nb_treeview_cb_selection_changed (GtkTreeSelection * selection, gpointer user_data)
{
	extern NbNotebook * notebook;
	gint count;
	gboolean same_note = FALSE;

	if ( notebook->note_edited_lock )
		return ;

	count = gtk_tree_selection_count_selected_rows (selection);

	if ( count > 1 )
		return;

	nb_notebook_save_note (notebook);

	gtk_tree_selection_selected_foreach (selection, (GtkTreeSelectionForeachFunc) nb_treeview_cb_update_notebook_with_current_note, (void *) same_note);

	/* if a new note has been added to the treeview we need to update the fields */
	if ( same_note  && ( notebook->note_is_inserted || notebook->note_is_modified ) )
		return ;

	/* TODO if ( nb_note_get_nb_attachments (g->current_note) > 0 )
		nb_ui_attachment_view_show ();
	else
		nb_ui_attachment_view_hide();

	nb_global_check_attachments (g);
	*/

	notebook->note_edited_lock = TRUE;

	nb_noteview_set_fields (notebook->ui->noteview, notebook->current_note);

	notebook->note_edited_lock = FALSE;
}

static void
nb_treeview_cb_update_notebook_with_current_note (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer user_data)
{
	extern NbNotebook * notebook;
	NbNote * n;
	gint index;
	gboolean same_note = (gboolean) user_data;

	gtk_tree_model_get (model, iter, NOTE_POINTER_COLUMN, &n, -1);

	if ( n == notebook->current_note )
	{
		same_note = TRUE;
		return ;
	}
	else if ( notebook->current_note != NULL )
		nb_note_clear_text (notebook->current_note);

	notebook->last_selected_note = notebook->current_note;
	notebook->current_note = n;

	/* load text from file */
	nb_notebook_load_note_text (notebook, notebook->current_note);

	notebook->note_is_inserted = TRUE;
}

/* column ordering callbacks */
static gint
nb_treeview_cb_sort_iter_compare_string (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gpointer userdata)
{
	gint column = GPOINTER_TO_INT (userdata);
	gchar * val1, * val2;
	gint ret;

	switch (column)
	{
		case TITLE_COLUMN:
			{
				gtk_tree_model_get (model, a, TITLE_COLUMN, &val1, -1);
				gtk_tree_model_get (model, b, TITLE_COLUMN, &val2, -1);
				break;
			}

		case TYPE_COLUMN:
			{
				gtk_tree_model_get (model, a, TYPE_COLUMN, &val1, -1);
				gtk_tree_model_get (model, b, TYPE_COLUMN, &val2, -1);
				break;
			}
	}

	if ( (val1 == NULL) || (val2 == NULL) )
	{
		if ( val1 == NULL )
		{
			if ( val2 == NULL )
				ret = 0;
			else
				ret = -1;
		}
		else
			ret = 1;
	}
	else
	{
		ret = g_utf8_collate (val1,val2);
	}

	g_free (val1);
	g_free (val2);

	return ret;
}

static gint
nb_treeview_cb_sort_iter_compare_date (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b, gpointer userdata)
{
	gint column = GPOINTER_TO_INT (userdata);
	NbNote * n1, * n2;
	gint d1, d2;

	gtk_tree_model_get (model, a, NOTE_POINTER_COLUMN, &n1, -1);
	gtk_tree_model_get (model, b, NOTE_POINTER_COLUMN, &n2, -1);

	if ( column == CREATION_DATE_COLUMN )
	{
		d1 = n1->creation_date;
		d2 = n2->creation_date;
	}
	else
	{
		d1 = n1->last_update;
		d2 = n2->last_update;
	}

	if ( d1 == d2 )
		return 0;
	else if ( d1 < d2 )
		return -1;
	else 
		return 1;
}

static gboolean
nb_treeview_cb_row_matches_query (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
	extern NbNotebook * notebook;
	NbNote * note;
	RowQueryList * parm = (RowQueryList *) data;

	gtk_tree_model_get ( GTK_TREE_MODEL (model), iter, NOTE_POINTER_COLUMN, &note, -1);
	
	nb_notebook_load_note_text (notebook, note);

	if ( !nb_note_matches_query (note, (gchar *) parm->query) )
	{
		GtkTreeRowReference  * rowref;

		rowref = gtk_tree_row_reference_new (model, path);
		parm->rr_list = g_list_append (parm->rr_list, rowref);

		note->iter = NULL;
	}

	return FALSE;
}

static void
nb_treeview_cb_drag_begin (GtkWidget * widget, GdkDragContext * context, gpointer user_data)
{
	extern NbNotebook * notebook;
	GtkTreeSelection * selection;

	if ( notebook->last_selected_note == NULL )
		return ;

	/* select the last_selected_note in the treeview while blocking the selection changed signal */
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (notebook->ui->treeview->tree));
	g_signal_handlers_block_by_func (G_OBJECT (selection), nb_treeview_cb_selection_changed, NULL);
	nb_treeview_select_iter (notebook->ui->treeview, notebook->last_selected_note->iter);
	g_signal_handlers_unblock_by_func (G_OBJECT (selection), nb_treeview_cb_selection_changed, NULL);

	nb_note_clear_text (notebook->current_note);

	notebook->dragged_row_index = nb_notebook_get_note_index (notebook, notebook->current_note);

	notebook->current_note = notebook->last_selected_note;

	/* load text from file */
	nb_notebook_load_note_text (notebook, notebook->current_note);

	notebook->note_edited_lock = TRUE;
	nb_noteview_set_fields (notebook->ui->noteview, notebook->current_note);
	notebook->note_edited_lock = FALSE;

	notebook->last_selected_note = NULL;
}

static gboolean
nb_treeview_cb_button_release (GtkWidget * widget, GdkEventButton * event, gpointer user_data)
{
	extern NbNotebook * notebook;
	notebook->last_selected_note = NULL;

	return FALSE;
}
